<?php

declare(strict_types=1);

namespace Drupal\jsonapi_request_logger\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Class that records detailed information of each JSON:API HTTP request.
 */
final class RequestLoggerSubscriber implements EventSubscriberInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  private ConfigFactoryInterface $configFactory;

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  private LoggerInterface $logger;

  /**
   * Constructor.
   */
  public function __construct(ConfigFactoryInterface $configFactory, LoggerInterface $logger) {
    $this->configFactory = $configFactory;
    $this->logger = $logger;
  }

  /**
   * Kernel request event handler.
   */
  public function onKernelRequest(RequestEvent $event): void {
    $config = $this->configFactory->get('jsonapi_request_logger.settings');
    $request = $event->getRequest();
    $method = $request->getMethod();

    // Check if logging is enabled.
    if (!$config->get('enable')) {
      return;
    }

    // Check if the request format is 'api_json'.
    if ($request->getRequestFormat() !== 'api_json') {
      return;
    }

    // Exclude routes based on the configured patterns.
    $exclude_routes = preg_split('/\r\n|\r|\n/', $config->get('exclude_routes'));
    $current_path = $request->getPathInfo();
    foreach ($exclude_routes as $pattern) {
      if (substr($pattern, -1) === '*') {
        $pattern = rtrim($pattern, '*');
        if (strpos($current_path, $pattern) === 0) {
          return;
        }
      }
      elseif ($pattern === $current_path) {
        return;
      }
    }

    // Filter by request methods.
    $allowed_methods = $config->get('request_methods') ?: [];
    if (!empty($allowed_methods) && !in_array($method, $allowed_methods, true)) {
      return;
    }

    // Check if the request body is empty and logging is set to omit empty bodies.
    if ($config->get('omit_empty_body') && empty($request->getContent())) {
      return;
    }

    // Log the request.
    $context = [
      '{method}' => $method,
      '{url}' => $request->getUri(),
      '{status_code}' => $event->getResponse() ? $event->getResponse()->getStatusCode() : 'N/A',
      '{headers}' => json_encode($request->headers->all(), JSON_PRETTY_PRINT),
      '{query}' => $request->getQueryString(),
      '{content}' => $request->getContent(),
    ];

    // Prepare the log message using the template.
    $log_template = $config->get('log_output_template') ?? 'Method: {method}, URL: {url}, Status: {status_code}, Headers: {headers}, Query: {query}, Content: {content}';
    $message = strtr($log_template, $context);

    // Write the log entry.
    $this->logger->info($message);
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      KernelEvents::REQUEST => ['onKernelRequest'],
    ];
  }

}
